// @ myth - common - buffer.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef BUFFER_HEADER
#define BUFFER_HEADER

// @@ include

#include "common/config.h"
#include "common/string.h"

// @@ enum buffer_mode

enum buffer_mode 
{
    BUFFER_NONE = 0,
    BUFFER_READ = 1,
    BUFFER_WRITE = 2,
    BUFFER_RDWR = 3,
    BUFFER_EXCLUSIVE = 6
};

// @@ enum buffer_state

enum buffer_state
{
    BUFFER_ERROR = ERROR,
    BUFFER_SUCCES = SUCCES,
    BUFFER_EOF = 1
};

// @@ class BUFFER

class BUFFER
{
    int fd;			// File descriptor
    buffer_mode mode;		// Input or output in buffer
    char* in;			// Input buffer
    int current;		// Amount of input buffer used
    int max;			// Amount of input buffer read
    buffer_state input;		// Error status of input buffer
    char* out;			// Output buffer
    int pos;			// Amount of output buffer used
    err output;			// Error status of output buffer
    void create(buffer_mode mode); // Initialise buffer
    bool IsTouched;		// Anything written to me?
public:
    // * Send and Receive data
    void get(void* buffer, int size); // Get data from buffers
    void put(const void* buffer, int size); // Put data in buffer
    void flush();		// Send content of buffer to file.
    // * Check 
    inline bool eof()		// EndOfFile on input reached?
    { return input == BUFFER_EOF; }
    inline bool GoodInput()	// No errors on input?
    { return input == BUFFER_SUCCES && fd != ERROR; }
    inline bool GoodOutput()	// No errors on output?
    { return output == SUCCES && fd != ERROR; }
    bool CheckTouched()		// Check if anything writen since last
    { return IsTouched; }
    void ClearTouched()		// call to ClearTouched()
    { IsTouched = FALSE; }
    // * Information used by WAIT
    inline int fileno()	CONST_MEMBER // Give fd to WAIT for select
    { return fd; }
    inline bool empty()		// Any data waiting in the input buffer?
    { return in == NULL || current == max; }
    bool IsReady;		// True, iff data is ready from fd or buffer
    // * Create and delete
    void open(int fd);		// Assign file descriptor to buffer
    void open(STRING name);
    BUFFER(buffer_mode mode, STRING name);
    BUFFER(buffer_mode mode, int fd = ERROR);
    ~BUFFER();
};

// @@ end

#endif BUFFER_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
