// @ myth - server - global.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/error.h"
#include "common/config.h"
#include "common/parse.h"
#include "server/global.h"
#include "server/rnd.h"
#include "server/watch.h"
#include "common/format.h"

// @@ GLOBAL

// @@@ STRING GLOBAL::DungeonFile()

STRING GLOBAL::DungeonFile()
{
    return format("%s/%s/DUNGEON", HomeDirectory, LibraryDirectory);
}

// @@@ STRING GLOBAL::CaveBinaryFile(level_type type)

STRING GLOBAL::CaveBinaryFile(level_type type)
{
    return format("%s/%s/%s.lb",
		  HomeDirectory, LibraryDirectory, LevelToCave(type).string());
}

// @@@ STRING GLOBAL::CaveTextFile(level_type type)

STRING GLOBAL::CaveTextFile(level_type type)
{
    return format("%s/%s/%s.cave",
		  HomeDirectory, LibraryDirectory, LevelToCave(type).string());
}

// @@@ level_type GLOBAL::CaveToLevel(STRING name)

level_type GLOBAL::CaveToLevel(STRING name)
{
    CaveNames.reset();

    for(;;)
    {
	if(CaveNames.current() == name)
	    return CaveNames.where();
	if(CaveNames.remain() == 0)
	    return LEVEL_ERROR;

	CaveNames.next();
    }
    return LEVEL_ERROR;		// Not Reached!
}

// @@@ STRING GLOBAL::LevelToCave(level_type depth)

STRING GLOBAL::LevelToCave(level_type depth)
{
    CaveNames.reset(depth);
    return CaveNames.current();
}

// @@@ void GLOBAL::load(int argc, char** argv)

void GLOBAL::load(int argc, char** argv)
{
    ShutDownTime = 30 * 60L;

    // Parse arguments
    for(;;)
    {
	argv++;
	argc--;

	if(argc == 0)
	    break;

	if(isdigit(**argv))
	{
	    ShutDownTime = atol(*argv);

	    if(ShutDownTime == 0L) // 0 is never!
		ShutDownTime = 60 * 60 * 24 * 365L;
	}
	else
	{
	    (void) fprintf(stderr, "Unknown argument %s\n", *argv);
	    (void) fprintf(stderr, "Usage: mythd [ timeout ]\n");
	    bug.fatal("GLOBAL::load", format("Argument %s", *argv));
	}
    }
    PARSE parse(DungeonFile().string());

    if(!parse.good())
	bug.fatal("GLOBAL::load",
		  format("Can't open %s", DungeonFile().string()));

    // Skip initial comments, if any
    if(parse.LookAhead(INPUT_COMMENT))
	parse.skip(INPUT_COMMENT);

    // Read name of each cave
    while(parse.good())
    {
	CaveNames.append(parse.GetWord());
	parse.skip(INPUT_COMMENT);
    }
}

// @@@ GLOBAL::GLOBAL()

GLOBAL::GLOBAL()
{ }

// @@@ GLOBAL global

GLOBAL global;

// @@ WATCH watch

WATCH watch;

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
