// @ myth - server - ring.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef RING_HEADER
#define RING_HEADER

// @@ include

#include "server/thing.h"

// @@ RING_ADORNMENT

class RING_ADORNMENT : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 100; }
public:
    object_type type()
    { return RING_OF_ADORNMENT; }
    const char* MagicName()
    { return "adornment"; }
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_ADORNMENT(this, arg); }
    RING_ADORNMENT(BINARY* binary) : (binary) { }
    RING_ADORNMENT(PARSE* parse) : (parse) { }
    RING_ADORNMENT(RING_ADORNMENT* thing,
			 long count) : (thing, count) { }
    RING_ADORNMENT(long count = 1L) : (count) { }
};

// @@ RING_HEALTH

class RING_HEALTH : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 2000; }
public:
    object_type type()
    { return RING_OF_HEALTH; }
    const char* MagicName()
    { return "health"; }
    //
    // Use
    //
    void InsertAbility(MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_HEALTH(this, arg); }
    RING_HEALTH(BINARY* binary) : (binary) { }
    RING_HEALTH(PARSE* parse) : (parse) { }
    RING_HEALTH(RING_HEALTH* thing,
			 long count) : (thing, count) { }
    RING_HEALTH(long count = 1L) : (count) { }
};

// @@ RING_TELEPORTATION

class RING_TELEPORTATION : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 500; }
public:
    object_type type()
    { return RING_OF_TELEPORTATION; }
    const char* MagicName()
    { return "teleportation"; }
    //
    // Use
    //
    void InsertAbility(MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_TELEPORTATION(this, arg); }
    RING_TELEPORTATION(BINARY* binary) : (binary) { }
    RING_TELEPORTATION(PARSE* parse) : (parse) { }
    RING_TELEPORTATION(RING_TELEPORTATION* thing,
			 long count) : (thing, count) { }
    RING_TELEPORTATION(long count = 1L) : (count) { }
};

// @@ RING_VISION

class RING_VISION : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 500; }
public:
    object_type type()
    { return RING_OF_VISION; }
    const char* MagicName()
    { return "vision"; }
    //
    // Use
    //
    void InsertAbility(MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_VISION(this, arg); }
    RING_VISION(BINARY* binary) : (binary) { }
    RING_VISION(PARSE* parse) : (parse) { }
    RING_VISION(RING_VISION* thing,
			 long count) : (thing, count) { }
    RING_VISION(long count = 1L) : (count) { }
};

// @@ RING_STRENGTH

class RING_STRENGTH : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 500; }
public:
    object_type type()
    { return RING_OF_STRENGTH; }
    const char* MagicName()
    { return "strength"; }
    //
    // Use
    //
    void InsertAbility(MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_STRENGTH(this, arg); }
    RING_STRENGTH(BINARY* binary) : (binary) { }
    RING_STRENGTH(PARSE* parse) : (parse) { }
    RING_STRENGTH(RING_STRENGTH* thing,
			 long count) : (thing, count) { }
    RING_STRENGTH(long count = 1L) : (count) { }
};

// @@ RING_FIRE_RESISTANCE

class RING_FIRE_RESISTANCE : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 750; }
public:
    object_type type()
    { return RING_OF_FIRE_RESISTANCE; }
    const char* MagicName()
    { return "fire resistance"; }
    //
    // Use
    //
    cont absorb(int* power, damage_type* type, OBJECT*, MONSTER*, MONSTER*);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_FIRE_RESISTANCE(this, arg); }
    RING_FIRE_RESISTANCE(BINARY* binary) : (binary) { }
    RING_FIRE_RESISTANCE(PARSE* parse) : (parse) { }
    RING_FIRE_RESISTANCE(RING_FIRE_RESISTANCE* thing,
			 long count) : (thing, count) { }
    RING_FIRE_RESISTANCE(long count = 1L) : (count) { }
};

// @@ RING_COLD_RESISTANCE

class RING_COLD_RESISTANCE : public RING
{
    // Identification
    //
protected:
    int BaseValue()
    { return 750; }
public:
    object_type type()
    { return RING_OF_COLD_RESISTANCE; }
    const char* MagicName()
    { return "cold resistance"; }
    //
    // Use
    //
    cont absorb(int* power, damage_type* type, OBJECT*, MONSTER*, MONSTER*);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_COLD_RESISTANCE(this, arg); }
    RING_COLD_RESISTANCE(BINARY* binary) : (binary) { }
    RING_COLD_RESISTANCE(PARSE* parse) : (parse) { }
    RING_COLD_RESISTANCE(RING_COLD_RESISTANCE* thing,
			 long count) : (thing, count) { }
    RING_COLD_RESISTANCE(long count = 1L) : (count) { }
};

// @@ AMULET_YENDOR

class AMULET_YENDOR : public AMULET
{
    // Identification
    //
public:
    object_type type()
    { return AMULET_OF_YENDOR; }
    const char* MagicName()
    { return "yendor"; }
    //
    // Use
    //
    void InsertAbility(MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new AMULET_YENDOR(this, arg); }
    AMULET_YENDOR(BINARY* binary) : (binary) { }
    AMULET_YENDOR(PARSE* parse) : (parse) { }
    AMULET_YENDOR(AMULET_YENDOR* thing,
			 long count) : (thing, count) { }
    AMULET_YENDOR(long count = 1L) : (count) { }
};

// @@ end

#endif AMULET_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
