// @ myth - server - sleep.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/sleep.h"
#include "common/binary.h"
#include "server/player.h"
#include "common/page.h"

// @@ struct sleep_link

// @@@ sleep_link::sleep_link(id_type id, BINARY* binary)

sleep_link::sleep_link(id_type id, BINARY* binary)
{
    identity = id;
    
    name = binary->GetString();
    passwd = binary->GetString();
    depth = binary->GetShort();
}

// @@@ sleep_link::sleep_link(STRING who,

sleep_link::sleep_link(STRING who, STRING how, level_type type, id_type id)
{
    name = who;
    passwd = how;
    depth = type;
    identity = id;
}

// @@@ void sleep_link::save(BINARY* binary)

void sleep_link::save(BINARY* binary)
{
    SaveIdentity(binary, identity); // MUST BE FIRST

    if(name.length() >= MessageSize - 1)
	binary->PutString("<some long name>");
    else if(name.length() > 0)
	binary->PutString(name.string());
    else
	binary->PutString("<no name>");

    if(passwd.length() >= MessageSize - 1)
	binary->PutString("");	// Don't use to long passwords!
    else if(passwd.length() > 0)
	binary->PutString(passwd.string());
    else
	binary->PutString("");
    
    binary->PutShort(depth);
}

// @@ SLEEP

// @@@ void SLEEP::erase()

void SLEEP::erase()
{
    delete current();
    LIST::remove();
}

// @@@ void SLEEP::add(PLAYER* player)

void SLEEP::add(PLAYER* player)
{
    LIST::add(new sleep_link(STRING(player->name()), player->passwd(),
			     player->depth(), player->identity()));
}

// @@@ void SLEEP::remove(id_type id)    

void SLEEP::remove(id_type id)    
{
    reset();

    while(remain() > 0)
    {
	if(current()->identity == id)
	    erase();
	else
	    next();
    }
}

// @@@ bool SLEEP::used(STRING who)

bool SLEEP::used(STRING who)
{
    reset();

    while(remain() > 0)
    {
	if(current()->name == who)
	    return TRUE;

	next();
    }
    return FALSE;
}

// @@@ id_type SLEEP::identify(STRING who, STRING how)

id_type SLEEP::identify(STRING who, STRING how)
{
    reset();

    while(remain() > 0)
    {
	if(current()->name == who && current()->passwd == how)
	    return current()->identity;
	else
	    next();
    }
    return NoIdentity;
}

// @@@ level_type SLEEP::where(id_type id)

level_type SLEEP::where(id_type id)
{
    reset();

    while(remain() > 0)
    {
	if(current()->identity == id)
	    return current()->depth;
	else
	    next();
    }
    return LEVEL_ERROR;
}

// @@@ void SLEEP::page(PAGE* page)

void SLEEP::page(PAGE* page)
{
    if(length() == 0)
    {
	page->append("No sleeping players.");
	return;
    }
    else if(length() == 1)
	page->append("Sleeping player:");
    else
	page->append("Sleeping players:");
    
    page->append("-");

    reset();
    while(remain() > 0)
    {
	page->append(current()->name);
	next();
    }
}

// @@@ void SLEEP::save(BINARY* binary)

void SLEEP::save(BINARY* binary)
{
    reset();

    while(remain() > 0)
    {
	current()->save(binary);
	next();
    }
    SaveIdentity(binary, NoIdentity);
}

// @@@ void SLEEP::load(BINARY* binary)

void SLEEP::load(BINARY* binary)
{
    while(binary->GoodInput())
    {
	id_type id = LoadIdentity(binary);

	if(id == NoIdentity)
	    break;
	else
	    LIST::add(new sleep_link(id, binary));
    }
}

// @@@ SLEEP::~SLEEP()
//
// Clean up on exit.

SLEEP::~SLEEP()
{
    reset();

    while(length() > 0)
	erase();
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
