// @ myth - server - specify.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef SPECIFY_HEADER
#define SPECIFY_HEADER

// @@ include

#include "common/command.h"
#include "server/object.h"

struct MONSTER;

// @@ ROCK

enum rock_type
{ FLOOR, UNEVEN_FLOOR, VERY_SOFT_ROCK, SOFT_ROCK, ROCK, HARD_ROCK,
  VERY_HARD_ROCK, PERMANENT_ROCK, ROCK_END, ROCK_ERROR = ERROR };

inline bool IsRock(rock_type rock)
{ return rock > UNEVEN_FLOOR; }

extern const char* RockName(rock_type rock);

extern int RockStrength(rock_type rock);

// @@ SPECIFY

// @@@ SPECIFY

class SPECIFY
{
public:
    virtual bool check(OBJECT*);
};

// @@@ SPECIFY_ALL

class SPECIFY_ALL : public SPECIFY
{
public:
    bool check(OBJECT*)
    { return TRUE; }
};

// @@@ SPECIFY_LETTER

class SPECIFY_LETTER : public SPECIFY
{
    symbol_type symbol;
    char letter;
public:
    bool check(OBJECT* object);
    SPECIFY_LETTER(symbol_type type, char c);
};

// @@@ SPECIFY_OBJECT

class SPECIFY_OBJECT : public SPECIFY
{
    OBJECT* what;
public:
    bool check(OBJECT* object)
    { return object == what; }
    SPECIFY_OBJECT(OBJECT* object)
    { what = object; }
};

// @@@ SPECIFY_IDENTITY

class SPECIFY_IDENTITY : public SPECIFY
{
    id_type identity;
public:
    bool check(OBJECT* object)
    { return object->identity() == identity; }
    SPECIFY_IDENTITY(id_type id)
    { identity = id; }
};

// @@@ SPECIFY_TYPE

class SPECIFY_TYPE : public SPECIFY
{
    object_type type;
public:
    bool check(OBJECT* object)
    { return object->type() == type; }
    SPECIFY_TYPE(object_type t)
    { type = t; }
};

// @@@ SPECIFY_SYMBOL

class SPECIFY_SYMBOL : public SPECIFY
{
    symbol_type symbol;
public:
    bool check(OBJECT* object)
    { return object->symbol() == symbol; }
    SPECIFY_SYMBOL(symbol_type t)
    { symbol = t; }
};

// @@@ SPECIFY_WORN

class SPECIFY_WORN : public SPECIFY
{
    MONSTER* monster;
public:
    bool check(OBJECT* object);
    SPECIFY_WORN(MONSTER* m)
    { monster = m; }
};

// @@@ SPECIFY_PICK

class SPECIFY_PICK : public SPECIFY
{
    MONSTER* monster;
    long extra;
public:
    bool check(OBJECT* object);
    SPECIFY_PICK(MONSTER* m);
};

// @@@ SPECIFY_WANT

class SPECIFY_WANT : public SPECIFY
{
    MONSTER* monster;
public:
    bool check(OBJECT* object);
    SPECIFY_WANT(MONSTER* m)
    { monster = m; }
};

// @@ end

#endif SPECIFY_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
